#ifndef JX_UART_SEND_H_
#define JX_UART_SEND_H_

#ifdef __cplusplus
extern "C" {
#endif

// 串口发送消息最大长度
#define UART_SEND_MAX      32
#define UART_MSG_HEAD_LEN_UART2_RX  2
#define UART_MSG_FOOT_LEN_UART2_RX  2

// 串口发送消息号
#define U_MSG_HX      0
#define U_MSG_ZJYL      1
#define U_MSG_JSYL      2
#define U_MSG_ZXYL      3
#define U_MSG_ZDYL      4
#define U_MSG_BFYY      5
#define U_MSG_ZTYY      6
#define U_MSG_XYS      7
#define U_MSG_SYS      8
#define U_MSG_JTDH      9
#define U_MSG_GDDH      10
#define U_MSG_DKLY      11
#define U_MSG_GBLY      12
#define U_MSG_OUT      13

// 串口消息参数类型
typedef union {
  double d_double;
  int d_int;
  unsigned char d_ucs[8];
  char d_char;
  unsigned char d_uchar;
  unsigned long d_long;
  short d_short;
  float d_float;
}uart_param_t;

// 十六位整数转32位整数
void _int16_to_int32(uart_param_t* param) {
  if (sizeof(int) >= 4)
    return;
  unsigned long value = param->d_long;
  unsigned long sign = (value >> 15) & 1;
  unsigned long v = value;
  if (sign)
    v = 0xFFFF0000 | value;
  uart_param_t p;
  p.d_long = v;
  param->d_ucs[0] = p.d_ucs[0];
  param->d_ucs[1] = p.d_ucs[1];
  param->d_ucs[2] = p.d_ucs[2];
  param->d_ucs[3] = p.d_ucs[3];
}

// 浮点数转双精度 
void _float_to_double(uart_param_t* param) {
  if (sizeof(int) >= 4)
    return;
  unsigned long value = param->d_long;
  unsigned long sign = value >> 31;
  unsigned long M = value & 0x007FFFFF;
  unsigned long e =  ((value >> 23 ) & 0xFF) - 127 + 1023;
  uart_param_t p0, p1;  
  p1.d_long = ((sign & 1) << 31) | ((e & 0x7FF) << 20) | (M >> 3);
  p0.d_long = (M & 0x7) << 29;
  param->d_ucs[0] = p0.d_ucs[0];
  param->d_ucs[1] = p0.d_ucs[1];
  param->d_ucs[2] = p0.d_ucs[2];
  param->d_ucs[3] = p0.d_ucs[3];
  param->d_ucs[4] = p1.d_ucs[0];
  param->d_ucs[5] = p1.d_ucs[1];
  param->d_ucs[6] = p1.d_ucs[2];
  param->d_ucs[7] = p1.d_ucs[3];
}

// action: HX
void _uart_HX();

// action: ZJYL
void _uart_ZJYL();

// action: JSYL
void _uart_JSYL();

// action: ZXYL
void _uart_ZXYL();

// action: ZDYL
void _uart_ZDYL();

// action: BFYY
void _uart_BFYY();

// action: ZTYY
void _uart_ZTYY();

// action: XYS
void _uart_XYS();

// action: SYS
void _uart_SYS();

// action: JTDH
void _uart_JTDH();

// action: GDDH
void _uart_GDDH();

// action: DKLY
void _uart_DKLY();

// action: GBLY
void _uart_GBLY();

// action: OUT
void _uart_OUT();

#ifdef __cplusplus
}
#endif

#endif /*JX_UART_SEND_H_*/

